@@ -1,5 +1,3 @@
-from __future__ import absolute_import
-
 import sys
 import warnings
 
@@ -18,6 +16,7 @@ from ..element import Element
 from ..ndmapping import NdMapping, item_check, sorted_context
 from .interface import DataError, Interface
 from .pandas import PandasInterface
+from .util import finite_range
 
 
 class cuDFInterface(PandasInterface):
@@ -130,7 +129,7 @@ class cuDFInterface(PandasInterface):
         if column.dtype.kind == 'O':
             return np.NaN, np.NaN
         else:
-            return (column.min(), column.max())
+            return finite_range(column, column.min(), column.max())
 
 
     @classmethod
@@ -238,7 +237,6 @@ class cuDFInterface(PandasInterface):
                 mask &= new_mask
         return mask
 
-
     @classmethod
     def select(cls, dataset, selection_mask=None, **selection):
         df = dataset.data
@@ -247,18 +245,16 @@ class cuDFInterface(PandasInterface):
 
         indexed = cls.indexed(dataset, selection)
         if selection_mask is not None:
-            df = df[selection_mask]
+            df = df.iloc[selection_mask]
         if indexed and len(df) == 1 and len(dataset.vdims) == 1:
             return df[dataset.vdims[0].name].iloc[0]
         return df
 
-
     @classmethod
     def concat_fn(cls, dataframes, **kwargs):
         import cudf
         return cudf.concat(dataframes, **kwargs)
 
-
     @classmethod
     def add_dimension(cls, dataset, dimension, dim_pos, values, vdim):
         data = dataset.data.copy()
@@ -266,7 +262,6 @@ class cuDFInterface(PandasInterface):
             data[dimension.name] = values
         return data
 
-
     @classmethod
     def aggregate(cls, dataset, dimensions, function, **kwargs):
         data = dataset.data
@@ -287,7 +282,13 @@ class cuDFInterface(PandasInterface):
             if not hasattr(reindexed, agg):
                 raise ValueError('%s aggregation is not supported on cudf DataFrame.' % agg)
             agg = getattr(reindexed, agg)()
-            data = dict(((col, [v]) for col, v in zip(agg.index.values_host, agg.to_array())))
+            try:
+                data = dict(((col, [v]) for col, v in zip(agg.index.values_host, agg.to_numpy())))
+            except Exception:
+                # Give FutureWarning: 'The to_array method will be removed in a future cuDF release.
+                # Consider using `to_numpy` instead.'
+                # Seen in cudf=21.12.01
+                data = dict(((col, [v]) for col, v in zip(agg.index.values_host, agg.to_array())))
             df = util.pd.DataFrame(data, columns=list(agg.index.values_host))
 
         dropped = []
