@@ -1,5 +1,3 @@
-from __future__ import absolute_import, division, unicode_literals
-
 from collections import defaultdict
 
 import numpy as np
@@ -14,7 +12,7 @@ from bokeh.transform import jitter
 from ...core.data import Dataset
 from ...core.dimension import dimension_name
 from ...core.util import (
-    OrderedDict, basestring, dimension_sanitizer, isfinite
+    OrderedDict, dimension_sanitizer, isfinite
 )
 from ...operation import interpolate_curve
 from ...util.transform import dim
@@ -26,7 +24,7 @@ from .styles import (
     expand_batched_style, base_properties, line_properties, fill_properties,
     mpl_to_bokeh, rgb2hex
 )
-from .util import bokeh_version, categorize_array
+from .util import LooseVersion, bokeh_version, categorize_array
 
 
 class PointPlot(LegendPlot, ColorbarPlot):
@@ -40,11 +38,11 @@ class PointPlot(LegendPlot, ColorbarPlot):
 
     # Deprecated parameters
 
-    color_index = param.ClassSelector(default=None, class_=(basestring, int),
+    color_index = param.ClassSelector(default=None, class_=(str, int),
                                       allow_None=True, doc="""
         Deprecated in favor of color style mapping, e.g. `color=dim('color')`""")
 
-    size_index = param.ClassSelector(default=None, class_=(basestring, int),
+    size_index = param.ClassSelector(default=None, class_=(str, int),
                                      allow_None=True, doc="""
         Deprecated in favor of size style mapping, e.g. `size=dim('size')`""")
 
@@ -74,7 +72,7 @@ class PointPlot(LegendPlot, ColorbarPlot):
         data, mapping = {}, {}
         sdim = element.get_dimension(self.size_index)
         ms = style.get('size', np.sqrt(6))
-        if sdim and ((isinstance(ms, basestring) and ms in element) or isinstance(ms, dim)):
+        if sdim and ((isinstance(ms, str) and ms in element) or isinstance(ms, dim)):
             self.param.warning(
                 "Cannot declare style mapping for 'size' option and "
                 "declare a size_index; ignoring the size_index.")
@@ -190,7 +188,7 @@ class VectorFieldPlot(ColorbarPlot):
     arrow_heads = param.Boolean(default=True, doc="""
         Whether or not to draw arrow heads.""")
 
-    magnitude = param.ClassSelector(class_=(basestring, dim), doc="""
+    magnitude = param.ClassSelector(class_=(str, dim), doc="""
         Dimension or dimension value transform that declares the magnitude
         of each vector. Magnitude is expected to be scaled between 0-1,
         by default the magnitudes are rescaled relative to the minimum
@@ -210,13 +208,13 @@ class VectorFieldPlot(ColorbarPlot):
 
     # Deprecated parameters
 
-    color_index = param.ClassSelector(default=None, class_=(basestring, int),
+    color_index = param.ClassSelector(default=None, class_=(str, int),
                                       allow_None=True, doc="""
         Deprecated in favor of dimension value transform on color option,
         e.g. `color=dim('Magnitude')`.
         """)
 
-    size_index = param.ClassSelector(default=None, class_=(basestring, int),
+    size_index = param.ClassSelector(default=None, class_=(str, int),
                                      allow_None=True, doc="""
         Deprecated in favor of the magnitude option, e.g.
         `magnitude=dim('Magnitude')`.
@@ -244,7 +242,7 @@ class VectorFieldPlot(ColorbarPlot):
                 "and declare a size_index; ignoring the size_index.")
         elif size_dim:
             mag_dim = size_dim
-        elif isinstance(mag_dim, basestring):
+        elif isinstance(mag_dim, str):
             mag_dim = element.get_dimension(mag_dim)
 
         (x0, x1), (y0, y1) = (element.range(i) for i in range(2))
@@ -268,11 +266,10 @@ class VectorFieldPlot(ColorbarPlot):
         return magnitudes
 
     def _glyph_properties(self, *args):
-        properties = super(VectorFieldPlot, self)._glyph_properties(*args)
+        properties = super()._glyph_properties(*args)
         properties.pop('scale', None)
         return properties
 
-
     def get_data(self, element, ranges, style):
         input_scale = style.pop('scale', 1.0)
 
@@ -412,7 +409,6 @@ class CurvePlot(ElementPlot):
         return data, mapping, style
 
 
-
 class HistogramPlot(ColorbarPlot):
 
     selection_display = BokehOverlaySelectionDisplay(color_prop=['color', 'fill_color'])
@@ -433,6 +429,8 @@ class HistogramPlot(ColorbarPlot):
             x = element.kdims[0]
             values = element.dimension_values(1)
             edges = element.interface.coords(element, x, edges=True)
+            if hasattr(edges, 'compute'):
+                edges = edges.compute()
             data = dict(top=values, left=edges[:-1], right=edges[1:])
             self._get_hover_data(data, element)
         return (data, mapping, style)
@@ -443,8 +441,7 @@ class HistogramPlot(ColorbarPlot):
         s0 = min(s0, 0) if isfinite(s0) else 0
         s1 = max(s1, 0) if isfinite(s1) else 0
         ranges[ydim.name]['soft'] = (s0, s1)
-        return super(HistogramPlot, self).get_extents(element, ranges, range_type)
-
+        return super().get_extents(element, ranges, range_type)
 
 
 class SideHistogramPlot(HistogramPlot):
@@ -470,7 +467,7 @@ class SideHistogramPlot(HistogramPlot):
     """
 
     def __init__(self, *args, **kwargs):
-        super(SideHistogramPlot, self).__init__(*args, **kwargs)
+        super().__init__(*args, **kwargs)
         if self.invert_axes:
             self.default_tools.append('ybox_select')
         else:
@@ -504,7 +501,7 @@ class SideHistogramPlot(HistogramPlot):
         """
         Returns a Bokeh glyph object.
         """
-        ret = super(SideHistogramPlot, self)._init_glyph(plot, mapping, properties)
+        ret = super()._init_glyph(plot, mapping, properties)
         if not 'field' in mapping.get('fill_color', {}):
             return ret
         dim = mapping['fill_color']['field']
@@ -687,13 +684,13 @@ class SpikesPlot(SpikesMixin, ColorbarPlot):
 
     # Deprecated parameters
 
-    color_index = param.ClassSelector(default=None, class_=(basestring, int),
+    color_index = param.ClassSelector(default=None, class_=(str, int),
                                       allow_None=True, doc="""
         Deprecated in favor of color style mapping, e.g. `color=dim('color')`""")
 
     selection_display = BokehOverlaySelectionDisplay()
 
-    style_opts = base_properties + line_properties + ['cmap', 'palette'] 
+    style_opts = base_properties + line_properties + ['cmap', 'palette']
 
     _nonvectorized_styles = base_properties + ['cmap']
     _plot_methods = dict(single='segment')
@@ -701,7 +698,7 @@ class SpikesPlot(SpikesMixin, ColorbarPlot):
     def _get_axis_dims(self, element):
         if 'spike_length' in self.lookup_options(element, 'plot').options:
             return  [element.dimensions()[0], None, None]
-        return super(SpikesPlot, self)._get_axis_dims(element)
+        return super()._get_axis_dims(element)
 
     def get_data(self, element, ranges, style):
         dims = element.dimensions()
@@ -779,7 +776,7 @@ class BarPlot(BarsMixin, ColorbarPlot, LegendPlot):
 
     # Deprecated parameters
 
-    color_index = param.ClassSelector(default=None, class_=(basestring, int),
+    color_index = param.ClassSelector(default=None, class_=(str, int),
                                       allow_None=True, doc="""
         Deprecated in favor of color style mapping, e.g. `color=dim('color')`""")
 
@@ -797,7 +794,7 @@ class BarPlot(BarsMixin, ColorbarPlot, LegendPlot):
 
     def _axis_properties(self, axis, key, plot, dimension=None,
                          ax_mapping={'x': 0, 'y': 1}):
-        props = super(BarPlot, self)._axis_properties(axis, key, plot, dimension, ax_mapping)
+        props = super()._axis_properties(axis, key, plot, dimension, ax_mapping)
         if (not self.multi_level and not self.stacked and self.current_frame.ndims > 1 and
             ((not self.invert_axes and axis == 'x') or (self.invert_axes and axis =='y'))):
             props['separator_line_width'] = 0
@@ -822,14 +819,12 @@ class BarPlot(BarsMixin, ColorbarPlot, LegendPlot):
             xdims = element.kdims[0]
         return (xdims, element.vdims[0])
 
-
     def _get_factors(self, element, ranges):
         xvals, gvals = self._get_coords(element, ranges)
         if gvals is not None:
             xvals = [(x, g) for x in xvals for g in gvals]
         return ([], xvals) if self.invert_axes else (xvals, [])
 
-
     def get_stack(self, xvals, yvals, baselines, sign='positive'):
         """
         Iterates over a x- and y-values in a stack layer
@@ -852,12 +847,10 @@ class BarPlot(BarsMixin, ColorbarPlot, LegendPlot):
             tops.append(top)
         return bottoms, tops
 
-
     def _glyph_properties(self, *args, **kwargs):
-        props = super(BarPlot, self)._glyph_properties(*args, **kwargs)
+        props = super()._glyph_properties(*args, **kwargs)
         return {k: v for k, v in props.items() if k not in ['width', 'bar_width']}
 
-
     def _add_color_data(self, ds, ranges, style, cdim, data, mapping, factors, colors):
         cdata, cmapping = self._get_color_data(ds, ranges, dict(style),
                                                factors=factors, colors=colors)
@@ -866,7 +859,7 @@ class BarPlot(BarsMixin, ColorbarPlot, LegendPlot):
 
         # Enable legend if colormapper is categorical
         cmapper = cmapping['color']['transform']
-        legend_prop = 'legend_field' if bokeh_version >= '1.3.5' else 'legend'
+        legend_prop = 'legend_field' if bokeh_version >= LooseVersion('1.3.5') else 'legend'
         if ('color' in cmapping and self.show_legend and
             isinstance(cmapper, CategoricalColorMapper)):
             mapping[legend_prop] = cdim.name
@@ -885,7 +878,6 @@ class BarPlot(BarsMixin, ColorbarPlot, LegendPlot):
             else:
                 data[k][-1] = cd
 
-
     def get_data(self, element, ranges, style):
         # Get x, y, group, stack and color dimensions
         group_dim, stack_dim = None, None
@@ -931,7 +923,7 @@ class BarPlot(BarsMixin, ColorbarPlot, LegendPlot):
 
         y0, y1 = ranges.get(ydim.name, {'combined': (None, None)})['combined']
         if self.logy:
-            bottom = (ydim.range[0] or (10**(np.log10(y1)-2)) if y1 else 0.01)
+            bottom = (ydim.range[0] or (0.01 if y1 > 0.01 else 10**(np.log10(y1)-2)))
         else:
             bottom = 0
         # Map attributes to data
@@ -977,7 +969,7 @@ class BarPlot(BarsMixin, ColorbarPlot, LegendPlot):
         for i, (k, ds) in enumerate(grouped.items()):
             k = k[0] if isinstance(k, tuple) else k
             if group_dim:
-                gval = k if isinstance(k, basestring) else group_dim.pprint_value(k)
+                gval = k if isinstance(k, str) else group_dim.pprint_value(k)
             # Apply stacking or grouping
             if grouping == 'stacked':
                 for sign, slc in [('negative', (None, 0)), ('positive', (0, None))]:
@@ -992,7 +984,10 @@ class BarPlot(BarsMixin, ColorbarPlot, LegendPlot):
                     data['top'].append(ts)
                     data[xdim.name].append(xs)
                     data[stack_dim.name].append(slc_ds.dimension_values(stack_dim))
-                    if hover: data[ydim.name].append(ys)
+                    if hover:
+                        data[ydim.name].append(ys)
+                        for vd in slc_ds.vdims[1:]:
+                            data[vd.name].append(slc_ds.dimension_values(vd))
                     if not style_mapping:
                         self._add_color_data(slc_ds, ranges, style, cdim, data,
                                              mapping, factors, colors)
@@ -1005,13 +1000,13 @@ class BarPlot(BarsMixin, ColorbarPlot, LegendPlot):
                 data[ydim.name].append(ys)
                 if hover: data[xdim.name].append(xs)
                 if group_dim not in ds.dimensions():
-                    ds = ds.add_dimension(group_dim.name, ds.ndims, gval)
+                    ds = ds.add_dimension(group_dim, ds.ndims, gval)
                 data[group_dim.name].append(ds.dimension_values(group_dim))
             else:
                 data[xdim.name].append(ds.dimension_values(xdim))
                 data[ydim.name].append(ds.dimension_values(ydim))
 
-            if hover:
+            if hover and grouping != 'stacked':
                 for vd in ds.vdims[1:]:
                     data[vd.name].append(ds.dimension_values(vd))
 
@@ -1029,7 +1024,7 @@ class BarPlot(BarsMixin, ColorbarPlot, LegendPlot):
 
         for name, val in mapping.items():
             sanitized = None
-            if isinstance(val, basestring):
+            if isinstance(val, str):
                 sanitized = dimension_sanitizer(mapping[name])
                 mapping[name] = sanitized
             elif isinstance(val, dict) and 'field' in val:
