@@ -1,5 +1,3 @@
-from __future__ import division
-
 import operator
 import sys
 
@@ -10,7 +8,8 @@ import param
 
 from ..core.data import PandasInterface
 from ..core.dimension import Dimension
-from ..core.util import basestring, flatten, pd, resolve_dependent_value, unique_iterator
+from ..core.util import flatten, pd, resolve_dependent_value, unique_iterator
+
 
 def _maybe_map(numpy_fn):
     def fn(values, *args, **kwargs):
@@ -222,7 +221,7 @@ class dim(object):
         ops = []
         self._ns = np.ndarray
         self.coerce = kwargs.get('coerce', True)
-        if isinstance(obj, basestring):
+        if isinstance(obj, str):
             self.dimension = Dimension(obj)
         elif isinstance(obj, Dimension):
             self.dimension = obj
@@ -238,7 +237,7 @@ class dim(object):
         else:
             fn = None
         if fn is not None:
-            if not (isinstance(fn, function_types+(basestring,)) or
+            if not (isinstance(fn, function_types+(str,)) or
                     any(fn in funcs for funcs in self._all_funcs)):
                 raise ValueError('Second argument must be a function, '
                                  'found %s type' % type(fn))
@@ -258,7 +257,7 @@ class dim(object):
             return self.ops[-1]['fn']
 
     def __call__(self, *args, **kwargs):
-        if (not self.ops or not isinstance(self.ops[-1]['fn'], basestring) or
+        if (not self.ops or not isinstance(self.ops[-1]['fn'], str) or
             'accessor' not in self.ops[-1]['kwargs']):
             raise ValueError("Cannot call method on %r expression. "
                              "Only methods accessed via namspaces, "
@@ -272,11 +271,11 @@ class dim(object):
         return self.clone(self.dimension, self.ops[:-1]+[new_op])
 
     def __getattribute__(self, attr):
-        self_dict = super(dim, self).__getattribute__('__dict__')
+        self_dict = super().__getattribute__('__dict__')
         if '_ns' not in self_dict: # Not yet initialized
-            return super(dim, self).__getattribute__(attr)
+            return super().__getattribute__(attr)
         ns = self_dict['_ns']
-        ops = super(dim, self).__getattribute__('ops')
+        ops = super().__getattribute__('ops')
         if ops and ops[-1]['kwargs'].get('accessor'):
             try:
                 ns = getattr(ns, ops[-1]['fn'])
@@ -289,7 +288,7 @@ class dim(object):
         if attr in extras and attr not in super(dim, self).__dir__():
             return type(self)(self, attr, accessor=True)
         else:
-            return super(dim, self).__getattribute__(attr)
+            return super().__getattribute__(attr)
 
     def __dir__(self):
         ns = self._ns
@@ -513,7 +512,7 @@ class dim(object):
         """
         args = list(args) # make mutable
         for k, arg in enumerate(args):
-            if isinstance(arg, basestring):
+            if isinstance(arg, str):
                 args[k] = cls(arg)
         return cls(args[0], func, *args[1:], **kwargs)
 
@@ -571,7 +570,7 @@ class dim(object):
                 kwargs['axis'] = None
             fn = fn_name
 
-        if isinstance(fn, basestring):
+        if isinstance(fn, str):
             accessor = kwargs.pop('accessor', None)
             fn_args = []
         else:
@@ -614,7 +613,7 @@ class dim(object):
         if (((fn is norm) or (fn is lognorm)) and drange != {} and
             not ('min' in kwargs and 'max' in kwargs)):
             data = fn(data, *drange)
-        elif isinstance(fn, basestring):
+        elif isinstance(fn, str):
             method = getattr(data, fn, None)
             if method is None:
                 mtype = 'attribute' if accessor else 'method'
@@ -773,14 +772,14 @@ class dim(object):
                 fn_name = self._unary_funcs[fn]
                 format_string = '{fn}' + prev
             else:
-                if isinstance(fn, basestring):
+                if isinstance(fn, str):
                     fn_name = fn
                 else:
                     fn_name = fn.__name__
                 if fn in self._builtin_funcs:
                     fn_name = self._builtin_funcs[fn]
                     format_string = '{fn}'+prev
-                elif isinstance(fn, basestring):
+                elif isinstance(fn, str):
                     if accessor:
                         sep = '' if op_repr.endswith(')') or prev_accessor else ')'
                         format_string = prev+sep+'.{fn}'
@@ -848,7 +847,7 @@ class df_dim(dim):
     _accessor = 'pd'
 
     def __init__(self, obj, *args, **kwargs):
-        super(df_dim, self).__init__(obj, *args, **kwargs)
+        super().__init__(obj, *args, **kwargs)
         self._ns = pd.Series
 
     def interface_applies(self, dataset, coerce):
@@ -891,7 +890,7 @@ class xr_dim(dim):
         except ImportError:
             raise ImportError("XArray could not be imported, dim().xr "
                               "requires the xarray to be available.")
-        super(xr_dim, self).__init__(obj, *args, **kwargs)
+        super().__init__(obj, *args, **kwargs)
         self._ns = xr.DataArray
 
     def interface_applies(self, dataset, coerce):
